package com.allmycode.hungryburds

import android.animation.Animator
import android.animation.Animator.AnimatorListener
import android.animation.AnimatorInflater
import android.animation.AnimatorSet
import android.annotation.SuppressLint
import android.content.Context
import android.content.SharedPreferences
import android.media.AudioAttributes
import android.media.SoundPool
import android.os.Bundle
import android.util.Size
import android.view.MotionEvent
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import androidx.appcompat.app.AppCompatActivity
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.constraintlayout.widget.ConstraintSet
import kotlinx.android.synthetic.main.activity_main.*
import java.util.Random


class MainActivity : AppCompatActivity(), View.OnTouchListener {
    private val numberOfBurds = 10
    private val maximumShowTime = 1000L
    private val minimumShowTime = 250L
    private val maximumImageEdge = 200
    private val minimumImageEdge = 50

    private var countShown = 0
    private var countTouched = 0
    private var highScore = 0
    private var random = Random()

    private lateinit var displaySize: Size

    private lateinit var prefs: SharedPreferences

    private lateinit var soundPool: SoundPool
    private var soundId = 0

    private var isStarted = false

    public override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        val metrics = resources.displayMetrics
        displaySize = Size(metrics.widthPixels, metrics.heightPixels)

        prefs = getPreferences(Context.MODE_PRIVATE)
        highScore = prefs.getInt("highScore", 0)
        displayHighScore()

        prepareSound()
    }

    override fun onStart() {
        super.onStart()
        isStarted = true
        showABurd()
    }

    private fun displayHighScore() {
        for (i in 1..highScore) {
            addBurgerTo(highScoreBurgersLayout)
        }
    }

    private fun addBurgerTo(layout: LinearLayout) {
        layout.addView(ImageView(this).apply {
            setImageResource(R.drawable.burger)
        })
    }

    private fun prepareSound() {
        soundPool = SoundPool.Builder()
                .setAudioAttributes(
                        AudioAttributes.Builder()
                                .setContentType(
                                        AudioAttributes.CONTENT_TYPE_SONIFICATION
                                )
                                .setUsage(AudioAttributes.USAGE_GAME)
                                .build()
                ).build()
        soundId = soundPool.load(this, R.raw.beep, 1)
    }

    private fun showABurd() {
        val duration = (random.nextInt((maximumShowTime
                - minimumShowTime).toInt()) + minimumShowTime)
        val imageEdgeSize = (random.nextInt(maximumImageEdge
                - minimumImageEdge) + minimumImageEdge)
        val burd = ImageView(this).also {
            it.setImageResource(R.drawable.burd)
            it.setOnTouchListener(this)
            it.layoutParams = ConstraintLayout.LayoutParams(
                    imageEdgeSize, imageEdgeSize
            )
        }
        addToScreen(burd)
        val animatorSet = AnimatorInflater.loadAnimator(
                this,
                R.animator.fade_in_out
        ) as AnimatorSet
        animatorSet.also {
            it.addListener(getListenerFor(burd))
            it.duration = duration
            it.setTarget(burd)
            it.start()
        }
    }

    private fun addToScreen(view: View) {
        view.id = View.generateViewId()
        screenLayout.addView(view, 0)
        ConstraintSet().apply {
            clone(screenLayout)
            connect(
                    view.id,
                    ConstraintSet.LEFT,
                    screenLayout.id,
                    ConstraintSet.LEFT,
                    random.nextInt(displaySize.width) * 7 / 8
            )
            connect(
                    view.id,
                    ConstraintSet.TOP,
                    screenLayout.id,
                    ConstraintSet.TOP,
                    random.nextInt(displaySize.height) * 4 / 5
            )
            applyTo(screenLayout)
        }
    }

    @SuppressLint("ClickableViewAccessibility")
    override fun onTouch(view: View?, event: MotionEvent?): Boolean {
        view?.setOnTouchListener(null)
        addBurgerTo(yourScoreBurgersLayout)
        soundPool.play(soundId, 1f, 1f, 0, 0, 1f)
        countTouched++
        return true
    }

    private fun getListenerFor(view: ImageView): AnimatorListener {
        return object : AnimatorListener {
            override fun onAnimationStart(animation: Animator?) {
            }

            override fun onAnimationRepeat(animation: Animator?) {
            }

            override fun onAnimationCancel(animation: Animator?) {
            }

            override fun onAnimationEnd(animation: Animator?) {
                screenLayout.removeView(view)
                if (isStarted) {
                    if (++countShown < numberOfBurds) {
                        showABurd()
                    } else {
                        saveHighScore()
                        playAgainButton.isEnabled = true
                        playAgainButton.isClickable = true
                    }
                }
            }
        }
    }

    private fun saveHighScore() {
        if (countTouched > highScore) {
            highScore = countTouched
            val editor = prefs.edit()
            editor.putInt("highScore", highScore)
            editor.apply()
        }
    }

    fun playAgain(view: View) {
        playAgainButton.isEnabled = false
        playAgainButton.isClickable = false
        countShown = 0
        countTouched = 0
        highScoreBurgersLayout.removeAllViews()
        displayHighScore()
        yourScoreBurgersLayout.removeAllViews()
        showABurd()
    }

    override fun onStop() {
        super.onStop()
        isStarted = false
    }
}
